#========================================================================================
# package      DE_EPAGES::Billsafe::t::unit::CheckoutPaymentValidation
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Billsafe::API::CheckoutPaymentValidation
#========================================================================================
package DE_EPAGES::Billsafe::t::unit::CheckoutPaymentValidation;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Billsafe::API::Account;
use DE_EPAGES::Billsafe::API::CheckoutPaymentValidation;
use DE_EPAGES::Billsafe::API::Common;
use DE_EPAGES::Billsafe::API::Request;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_inValidBasket();
    $self->test_validBasketSuccessWithInvoice();
    $self->test_validBasketSuccessWithoutInvoice();
    $self->test_validBasketNotSuccess();
    $self->test_validBasketAndErrorRequest();

    done_testing();
}


sub test_inValidBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $isErrorRequest = 1;
    my $InvalidBasketObject = {};
    my $Validation = $self->_createCheckoutPaymentValidation($InvalidBasketObject, $isErrorRequest);

    is($Validation->isValidForRequest(), 0, "$SubTestName - Check the behaivor with an invalid basket");
}


sub test_validBasketSuccessWithInvoice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $MockedBillsafeRequest = $self->_createBillsafeRequestSuccessWithInvoice();
    my $Validation = $self->_createCheckoutPaymentValidation($Basket, $MockedBillsafeRequest);

    is($Validation->isValidForRequest(), 1, "$SubTestName - Check if the basket is valid for request");
    is($Validation->request(), undef, "$SubTestName - Check if the request is made and get nothing as result");

    my $InputOn_createBillsafeRequest = $Validation->{'LineItemPayment__createBillsafeRequest'};
    isa_ok($InputOn_createBillsafeRequest, 'DE_EPAGES::Billsafe::API::Object::LineItemPaymentBillsafe',
        "$SubTestName - Check if the LineItemPayment was handed over to the _createBillsafeRequest method");

    my $ExpectedInputOnprevalidateOrder = $self->_createExpectedhOrder();
    my $InputOnPrevalidateOrder = $MockedBillsafeRequest->{'hOrder_prevalidateOrder'};
    is_deeply($InputOnPrevalidateOrder, $ExpectedInputOnprevalidateOrder,
        "$SubTestName - Check if the hOrder was handed over to the prevalidateOrder method");

    is($Validation->hasError(), 0, "$SubTestName - Check if no error appears");
    is($Validation->getErrorMessage(), '', "$SubTestName - Check if no error message appears");
}


sub test_validBasketSuccessWithoutInvoice {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $MockedBillsafeRequest = $self->_createBillsafeRequestSuccessWithoutInvoice();
    my $Validation = $self->_createCheckoutPaymentValidation($Basket, $MockedBillsafeRequest);

    is($Validation->isValidForRequest(), 1, "$SubTestName - Check if the basket is valid for request");
    is($Validation->request(), undef, "$SubTestName - Check if the request is made and get nothing as result");

    my $ExpectedInputOnLogPayment = 'BillsafePrevalidateOrderFailed';
    my $InputOnLogPayment = $Validation->{'Message__logPayment'};
    is($InputOnLogPayment, $ExpectedInputOnLogPayment, "$SubTestName - Check if the logPayment was called in case of error");

    is($Validation->hasError(), 1, "$SubTestName - Check if the error appears (Response without Invoice)");
    is($Validation->getErrorMessage(), 'BillsafePrevalidateOrderFailed', "$SubTestName - Check if the error message appears (Response without Invoice)");


    my $InputOn_createBillsafeRequest = $Validation->{'LineItemPayment__createBillsafeRequest'};
    isa_ok($InputOn_createBillsafeRequest, 'DE_EPAGES::Billsafe::API::Object::LineItemPaymentBillsafe',
        "$SubTestName - Check if the LineItemPayment was handed over to the _createBillsafeRequest method");

    my $ExpectedInputOnprevalidateOrder = $self->_createExpectedhOrder();
    my $InputOnPrevalidateOrder = $MockedBillsafeRequest->{'hOrder_prevalidateOrder'};
    is_deeply($InputOnPrevalidateOrder, $ExpectedInputOnprevalidateOrder,
        "$SubTestName - Check if the hOrder was handed over to the prevalidateOrder method");
}


sub test_validBasketNotSuccess {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $MockedBillsafeRequest = $self->_createBillsafeRequestNotSuccess();
    my $Validation = $self->_createCheckoutPaymentValidation($Basket, $MockedBillsafeRequest);

    is($Validation->isValidForRequest(), 1, "$SubTestName - Check if the basket is valid for request");
    is($Validation->request(), undef, "$SubTestName - Check if the request is made and get nothing as result");

    my $ExpectedInputOnLogPayment = 'Response not sucessful';
    my $InputOnLogPayment = $Validation->{'Message__logPayment'};
    is($InputOnLogPayment, $ExpectedInputOnLogPayment, "$SubTestName - Check if the logPayment was called in case of error");

    is($Validation->hasError(), 1, "$SubTestName - Check if the error appears (Response not sucessful)");
    is($Validation->getErrorMessage(), 'BillsafePrevalidateOrderFailed', "$SubTestName - Check if the error message appears (Response not sucessful)");


    my $InputOn_createBillsafeRequest = $Validation->{'LineItemPayment__createBillsafeRequest'};
    isa_ok($InputOn_createBillsafeRequest, 'DE_EPAGES::Billsafe::API::Object::LineItemPaymentBillsafe',
        "$SubTestName - Check if the LineItemPayment was handed over to the _createBillsafeRequest method");

    my $ExpectedInputOnprevalidateOrder = $self->_createExpectedhOrder();
    my $InputOnPrevalidateOrder = $MockedBillsafeRequest->{'hOrder_prevalidateOrder'};
    is_deeply($InputOnPrevalidateOrder, $ExpectedInputOnprevalidateOrder,
        "$SubTestName - Check if the hOrder was handed over to the prevalidateOrder method");
}


sub test_validBasketAndErrorRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasket();
    my $MockedBillsafeRequest = $self->_createBillsafeRequestThrowingError();
    my $Validation = $self->_createCheckoutPaymentValidation($Basket, $MockedBillsafeRequest);

    is($Validation->isValidForRequest(), 1, "$SubTestName - Check if the basket is valid for request");
    is($Validation->request(), undef, "$SubTestName - Check if the request is made and get nothing as result");

    my $ExpectedInputOnLogError = 'MyTestError';
    my $InputOnLogError = $Validation->{'Error__logError'}->code();
    is($InputOnLogError, $ExpectedInputOnLogError, "$SubTestName - Check if the error was logged");

    is($Validation->hasError(), 1, "$SubTestName - Check if the error appear (Error was throw)");
    is($Validation->getErrorMessage(), 'BillsafePrevalidateOrderFailed', "$SubTestName - Check if the error message appears (Error was throw)");
}


sub _createExpectedhOrder {
    my $self = shift;
    return {
          'order_amount' => '99.99',
          'order_currencyCode' => 'EUR',
          'deliveryAddress_country' => 'DE',
          'deliveryAddress_houseNumber' => '2',
          'deliveryAddress_street' => 'Test Strasse',
          'deliveryAddress_postcode' => '20355',
          'deliveryAddress_city' => 'Hamburg',
          'deliveryAddress_firstname' => 'Test',
          'deliveryAddress_lastname' => 'Benutzer'
    };
}


sub TestExistsMethod {
    my ($PathOrObject, $MethodName) = @_;
    if( not $PathOrObject->can($MethodName)){
        Error(ref($PathOrObject)." don't have a method like: $MethodName");
    }
    return;
}

#---------------------------------------------------------------------------
#  Mocked Epages Objects
#---------------------------------------------------------------------------
sub _createCheckoutPaymentValidation {
    my $self = shift;
    my ($Basket, $BillsafeRequest) = @_;

    my $Validation = DE_EPAGES::Billsafe::API::CheckoutPaymentValidation->new($Basket);
    my $MockedValidation = Test::MockObject::Extends->new($Validation);
    TestExistsMethod($Validation, '_createBillsafeRequest');
    $MockedValidation->mock('_createBillsafeRequest', sub {
        my $MockedSelf = shift;
        my ($LineItemPayment) = @_;
        $MockedValidation->{'LineItemPayment__createBillsafeRequest'} = $LineItemPayment;
        return $BillsafeRequest;
    });
    TestExistsMethod($Validation, '_logError');
    $MockedValidation->mock('_logError', sub {
        my $MockedSelf = shift;
        my ($Error) = @_;
        $MockedValidation->{'Error__logError'} = $Error;
        return;
    });
    TestExistsMethod($Validation, '_logPayment');
    $MockedValidation->mock('_logPayment', sub {
        my $MockedSelf = shift;
        my ($Cartridge, $Message, $hParams) = @_;
        $MockedValidation->{'Message__logPayment'} = $Message;
        return;
    });

    return $MockedValidation;
}

sub _createBillsafeRequestSuccessWithInvoice {
    my $self = shift;

    my $BillsafeRequest = $self->_createBillsafeRequest();
    my $IsSuccessResponse = 1;

    $BillsafeRequest->mock('request', sub {
        my $MockedSelf = shift;
        return $self->_createBillsafeResponse($IsSuccessResponse);
    });

    $BillsafeRequest->mock('parse_nvp', sub {
        my $MockedSelf = shift;
        my ($Response) = @_;
        $BillsafeRequest->{'Response_parse_nvp'} = $Response;
        my $hResult = {'invoice_isAvailable' => 'TRUE'};
        return %{$hResult};
    });

    return $BillsafeRequest;
}


sub _createBillsafeRequestSuccessWithoutInvoice {
    my $self = shift;

    my $BillsafeRequest = $self->_createBillsafeRequest();
    my $IsSuccessResponse = 1;

    $BillsafeRequest->mock('request', sub {
        my $MockedSelf = shift;
        return $self->_createBillsafeResponse($IsSuccessResponse);
    });

    $BillsafeRequest->mock('parse_nvp', sub {
        my $MockedSelf = shift;
        my ($Response) = @_;
        $BillsafeRequest->{'Response_parse_nvp'} = $Response;
        my $hResult = {'invoice_isAvailable' => 'FALSE'};
        return %{$hResult};
    });

    return $BillsafeRequest;
}


sub _createBillsafeRequestNotSuccess {
    my $self = shift;

    my $BillsafeRequest = $self->_createBillsafeRequest();
    my $IsSuccessResponse = 0;

    $BillsafeRequest->mock('request', sub {
        my $MockedSelf = shift;
        return $self->_createBillsafeResponse($IsSuccessResponse);
    });

    return $BillsafeRequest;
}

sub _createBillsafeRequestThrowingError {
    my $self = shift;

    my $BillsafeRequest = $self->_createBillsafeRequest();

    $BillsafeRequest->mock('request', sub {
        my $MockedSelf = shift;
        Error('MyTestError');
        return;
    });

    return $BillsafeRequest;
}


sub _createBillsafeRequest {
    my $self = shift;

    TestExistsMethod('DE_EPAGES::Billsafe::API::Account', 'epages');
    TestExistsMethod('DE_EPAGES::Billsafe::API::Common', 'LogBillsafe');
    TestExistsMethod('DE_EPAGES::Billsafe::API::Request', 'new');
    TestExistsMethod('DE_EPAGES::Billsafe::API::Request', 'prevalidateOrder');
    TestExistsMethod('DE_EPAGES::Billsafe::API::Request', 'request');
    TestExistsMethod('DE_EPAGES::Billsafe::API::Request', 'parse_nvp');

    my $MockedBillsafeRequest = Test::MockObject->new();
    $MockedBillsafeRequest->mock('prevalidateOrder', sub {
        my $MockedSelf = shift;
        my ($hOrder) = @_;
        $MockedBillsafeRequest->{'hOrder_prevalidateOrder'} = $hOrder;
        return;
    });

    return $MockedBillsafeRequest;
}


sub _createBillsafeResponse {
    my $self = shift;
    my ($IsSucess) = @_;
    my $Response = Test::MockObject->new();
    $Response->mock('is_success', sub { return $IsSucess; });
    return $Response;
}


sub _createBasket {
    my $self = shift;
    my $hAtributes = {
        'Alias' => 'MyTestBasket',
        'BillingAddress' => $self->_createBillingAddress(),
        'LineItemContainer' => $self->_createLineItemContainer(),
    };
    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket',$hAtributes);
    return $Basket;
}

sub _createBillingAddress {
    my $self = shift;
    my $hAtributes = {
        'Alias' => 'BillingAddress',
        'FirstName' => 'Test',
        'LastName' => 'Benutzer',
        'StreetOnly' => 'Test Strasse',
        'HouseNumber' => '2',
        'Zipcode' => '20355',
        'City' => 'Hamburg',
        'Country' => {'Code2' => 'DE'},
    };
    my $BillingAddress = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address',$hAtributes);
    return $BillingAddress;
}


sub _createLineItemContainer {
    my $self = shift;
    my $hAtributes = {
        'Alias' => 'MyTestLineItemContainer',
        'Payment' => $self->_createLineItemPayment()
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer',$hAtributes);
    return $LineItemContainer;
}


sub _createLineItemPayment {
    my $self = shift;
    my $hAtributes = {
        'Alias' => 'LineItemPaymentBillsafe',
        'Amount' => '99.99',
        'CurrencyID' => 'EUR',
        'PaymentMethod' => $self->_createPaymentMethod(),
    };
    my $LineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Billsafe::API::Object::LineItemPaymentBillsafe',$hAtributes);
    return $LineItemPayment;
}


sub _createPaymentMethod {
    my $self = shift;
    my $hAtributes = {
        'Alias' => 'PaymentMethodBillsafe',
        'MerchantName' => 'TestMechant',
        'MerchantPassword' => 'TestPassword',
    };
    my $PaymentMethod = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Billsafe::API::Object::PaymentMethodBillsafe',$hAtributes);
    return $PaymentMethod;
}


__PACKAGE__->runScript();
1;